      subroutine nnamd_qp(ip,mu,q,e0,ub,u,e,mat)
c----------------------------------------------------------------------c
c Returns the matrix element bu*e_n*Gamma^n*u with
c    Gamma^n=e0*[F_1(q^2)*gamma^n+F_2(q^2)*i*sigma^nm*q_m]
c representing an effective nucleon-nucleon-photon coupling. 
c Factor 1/(2m) included in F_2. Based on a routine btwmd.f.
c
c Input: 
c        ip = 1/-1 if bar spinor u represents a particle/antiparticle,
c        mu - mass of a nucleon represented by u and ub,
c        q - four momentum of incoming e,
c        e0 - electric charge,
c        ub,u - nucleon spinors,
c        e -  polarization vector of a photon.
c
c The Dirac gamma matrices are in the Weyl representation
c
c Karol Kolodziej, University of Silesia, karol.kolodziej@us.edu.pl
c 19 Feb./13 March, 2014
c 6 March, 2020: 
c quadruple precision version written
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0), qp=kind(1.q0)

      integer :: ip
      real(qp), dimension(0:3) :: q,pe
      complex(qp) :: mat
      complex(qp), dimension(4) :: ub,u
      complex(qp), dimension(0:3) :: e
      real(dp) :: mu,e0
      complex(dp) :: f1,f2

c Local variables:

      integer :: l
      real(qp) :: den
      complex(qp), parameter :: icom=(0.q0,1.q0)
      complex(dp) :: f1l,f1r,f2l,f2r
      complex(qp), dimension(0:3,2,2) :: a,d
      complex(qp), dimension(0:3) :: ew

      den=q(0)**2-q(1)**2-q(2)**2-q(3)**2

      call nuclff_phok(mu,den,f1,f2)

      f1r=e0*f1
      f1l=f1r
      f2r=e0*f2
      f2l=f2r

      if (ip == -1) then
       f1r=conjg(f1r)
       f1l=conjg(f1l)
       f2r=conjg(f2r)
       f2l=conjg(f2l)
      end if

      pe=-q ! Vertex is parametrized in terms of outgoing photon four momentum

      a(0,1,:)=(/cmplx(pe(3),0.q0,qp),cmplx(pe(1),-pe(2),qp)/)
      a(0,2,:)=(/cmplx(pe(1),pe(2),qp),-a(0,1,1)/)
      a(1,1,:)=(/cmplx(0.q0,pe(2),qp),cmplx(pe(0)-pe(3),0.q0,qp)/)
      a(1,2,:)=(/cmplx(pe(0)+pe(3),0.q0,qp),-a(1,1,1)/)
      a(2,1,:)=(/cmplx(0.q0,-pe(1),qp),cmplx(0.q0,-pe(0)+pe(3),qp)/)
      a(2,2,:)=(/cmplx(0.q0,pe(0)+pe(3),qp),-a(2,1,1)/)
      a(3,1,:)=(/cmplx(pe(0),0.q0,qp),a(0,1,2)/)
      a(3,2,:)=(/-a(0,2,1),-a(3,1,1)/)
      d(0,:,:)=-a(0,:,:)
      d(1,1,:)=(/a(1,1,1),-a(1,2,1)/)
      d(1,2,:)=(/-a(1,1,2),-a(1,1,1)/)
      d(2,1,:)=(/a(2,1,1),a(2,2,1)/)
      d(2,2,:)=(/a(2,1,2),a(2,2,2)/)
      d(3,1,:)=(/a(3,2,2),a(3,1,2)/)
      d(3,2,:)=(/a(3,2,1),a(3,1,1)/)

      do l=0,3
       ew(l)=f2l*(ub(1)*(a(l,1,1)*u(1)+a(l,1,2)*u(2))
     &           +ub(2)*(a(l,2,1)*u(1)+a(l,2,2)*u(2)))
     &      +f2r*(ub(3)*(d(l,1,1)*u(3)+d(l,1,2)*u(4))
     &           +ub(4)*(d(l,2,1)*u(3)+d(l,2,2)*u(4)))
      end do
      ew(0)=ew(0)+f1l*(ub(3)*u(1)+ub(4)*u(2))
     &           +f1r*(ub(1)*u(3)+ub(2)*u(4))
      ew(1)=ew(1)-f1l*(ub(3)*u(2)+ub(4)*u(1))
     &           +f1r*(ub(1)*u(4)+ub(2)*u(3))
      ew(2)=ew(2)-f1l*(-ub(3)*u(2)+ub(4)*u(1))*icom
     &           +f1r*(-ub(1)*u(4)+ub(2)*u(3))*icom
      ew(3)=ew(3)-f1l*(ub(3)*u(1)-ub(4)*u(2))
     &           +f1r*(ub(1)*u(3)-ub(2)*u(4))

      mat=ew(0)*e(0)-ew(1)*e(1)-ew(2)*e(2)-ew(3)*e(3)

      end




