      subroutine recpol_qp(p,mass,imc,phi,u,npol)
c----------------------------------------------------------------------c
c Returns real polarization vectors in the rectangular basis.
c Input: p(0:3) - particle four mometum,
c        mass - particle mass.
c
c The polarization vectors are defined as in the paper by K. Kolodziej, 
c M. Zralek, Phys. Rev. D43 (1991) 3619:
c
c If imc=1 then polarizations are summed
c        exp(i*phi)*u(:,1) + exp(-i*phi)*u(:,2),  for a massless boson
c                    - " -              +u(:,3),  for a massive boson
c else
c   2 or 3 polarization vectors are returned for a massless or massive 
c   boson, respectively.
c
c Karol Kolodziej, University of Silesia, kolodzie@us.edu.pl
c 4 March, 1998;  modified on 22 March, 2006 (interface changed)
c                              6 May (interface changed again)
c                             19 October, 2007 (MC summing over 
c                                            polarizations implemented)
c 30 Jan. 2019: quadurple precision four momenta implemented
c 25 Feb. 2021: QP version modified
c----------------------------------------------------------------------c

      implicit none

      integer, parameter :: dp=kind(1.d0),qp=kind(1.q0)
      integer :: imc,npol

      real(dp), parameter :: zero=0.d0,one=1.d0
      real(qp) :: p(0:3)
      real(dp) :: mass,phi
      real(qp) :: eps(0:3,3),gamma,ct,st,cf,sf,pp,pt
      complex(qp) :: u(0:3,npol)

      pt=sqrt(p(1)**2+p(2)**2)

      if (pt == 0.q0) then

       pp=abs(p(3))
       ct=p(3)/pp
       st=0.q0
       cf=1.q0
       sf=0.q0

      else

       pp=sqrt(pt**2+p(3)**2)
       ct=p(3)/pp
       st=sqrt(1.q0-ct**2)
       cf=p(1)/pt
       sf=p(2)/pt

      end if



      if (abs(ct) .lt. 1.q0) then

      eps(:,1)=(/ 0.q0,
     &            ct*cf,
     &            ct*sf,
     &           -st /)

      eps(:,2)=(/ 0.q0,
     &           -sf,
     &            cf,
     &            0.q0 /)


      if (mass.gt.zero) then
        gamma=p(0)/mass
        eps(:,3)=(/ pp/mass,
     &             gamma*st*cf,
     &             gamma*st*sf,
     &             gamma*ct /)
      end if

      else if (ct.eq.1.q0) then

      eps(:,1)=(/ 0.q0,
     &            1.q0,
     &            0.q0,
     &            0.q0 /)

      eps(:,2)=(/ 0.q0,
     &            0.q0,
     &            1.q0,
     &            0.q0 /)


      if (mass.gt.zero) then
        eps(:2,3)=(/ pp/mass,
     &               0.q0,
     &               0.q0/)
        eps(3,3)=p(0)/mass
      end if

      else if (ct.eq.-1.q0) then

      eps(:,1)=(/ 0.q0,
     &            1.q0,
     &            0.q0,
     &            0.q0 /)

      eps(:,2)=(/ 0.q0,
     &            0.q0,
     &            -1.q0,
     &            0.q0/)

      if (mass.gt.zero) then
        eps(:2,3)=(/ pp/mass,
     &               0.q0,
     &               0.q0/)
        eps(3,3)=-p(0)/mass
      end if

      end if


      if (imc == 1) then
       if (mass > 0.d0) then
        u(:,1)=cmplx(cos(phi)*(eps(:,1)+eps(:,2))+eps(:,3),
     &               sin(phi)*(eps(:,1)-eps(:,2)),qp)
       else
        u(:,1)=cmplx(cos(phi)*(eps(:,1)+eps(:,2)),
     &               sin(phi)*(eps(:,1)-eps(:,2)),qp)
       end if
      else
       u=cmplx(eps(:,1:npol),0.q0,qp)
      end if


      end subroutine

